@@ -3,7 +3,7 @@
  * Low-level USB interface glue towards libusb.
  *
  * Copyright (C) 2005-2007 Richard A. Low <richard@wentnet.com>
- * Copyright (C) 2005-2011 Linus Walleij <triad@df.lth.se>
+ * Copyright (C) 2005-2012 Linus Walleij <triad@df.lth.se>
  * Copyright (C) 2006-2011 Marcus Meissner
  * Copyright (C) 2007 Ted Bullock
  * Copyright (C) 2008 Chris Bagwell <chris@cnpbagwell.com>
@@ -93,7 +93,9 @@ static const int mtp_device_table_size = sizeof (mtp_device_table) / sizeof (LIB
 static void init_usb();
 static void close_usb(PTP_USB* ptp_usb);
 static int find_interface_and_endpoints(openusb_dev_handle_t *dev,
+        uint8_t *conf,
         uint8_t *interface,
+        uint8_t *altsetting,
         int* inep,
         int* inep_maxpacket,
         int* outep,
@@ -659,17 +661,7 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
             }
         }
         if (!device_known) {
-            // This device is unknown to the developers
-            LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is UNKNOWN.\n",
-                    i,
-                    desc.idVendor,
-                    desc.idProduct);
-            LIBMTP_ERROR("Please report this VID/PID and the device model to the libmtp development team\n");
-            /*
-             * Trying to get iManufacturer or iProduct from the device at this
-             * point would require opening a device handle, that we don't want
-             * to do right now. (Takes time for no good enough reason.)
-             */
+            device_unknown(i, desc.idVendor, desc.idProduct);
         }
         // Save the location on the bus
         retdevs[i].bus_location = 0;
@@ -801,7 +793,23 @@ ptp_read_func(
     unsigned long written;
     unsigned char *bytes;
     int expect_terminator_byte = 0;
+    unsigned long usb_inep_maxpacket_size;
+    unsigned long context_block_size_1;
+    unsigned long context_block_size_2;
+    uint16_t ptp_dev_vendor_id = ptp_usb->rawdevice.device_entry.vendor_id;
 
+    //"iRiver" device special handling
+    if (ptp_dev_vendor_id == 0x4102 || ptp_dev_vendor_id == 0x1006) {
+	    usb_inep_maxpacket_size = ptp_usb->inep_maxpacket;
+	    if (usb_inep_maxpacket_size == 0x400) {
+		    context_block_size_1 = CONTEXT_BLOCK_SIZE_1 - 0x200;
+		    context_block_size_2 = CONTEXT_BLOCK_SIZE_2 + 0x200;
+	    }
+	    else {
+		    context_block_size_1 = CONTEXT_BLOCK_SIZE_1;
+		    context_block_size_2 = CONTEXT_BLOCK_SIZE_2;
+	    }
+    }
     struct openusb_bulk_request bulk;
     // This is the largest block we'll need to read in.
     bytes = malloc(CONTEXT_BLOCK_SIZE);
@@ -818,16 +826,21 @@ ptp_read_func(
                 toread += 1;
                 expect_terminator_byte = 1;
             }
-        } else if (curread == 0)
-            // we are first packet, but not last packet
-            toread = CONTEXT_BLOCK_SIZE_1;
-        else if (toread == CONTEXT_BLOCK_SIZE_1)
-            toread = CONTEXT_BLOCK_SIZE_2;
-        else if (toread == CONTEXT_BLOCK_SIZE_2)
-            toread = CONTEXT_BLOCK_SIZE_1;
-        else
-            LIBMTP_INFO("unexpected toread size 0x%04x, 0x%04x remaining bytes\n",
-                (unsigned int) toread, (unsigned int) (size - curread));
+        } else if (ptp_dev_vendor_id == 0x4102 || ptp_dev_vendor_id == 0x1006) {
+		//"iRiver" device special handling
+		if (curread == 0)
+			// we are first packet, but not last packet
+			toread = context_block_size_1;
+		else if (toread == context_block_size_1)
+			toread = context_block_size_2;
+		else if (toread == context_block_size_2)
+			toread = context_block_size_1;
+		else
+			LIBMTP_INFO("unexpected toread size 0x%04x, 0x%04x remaining bytes\n",
+				    (unsigned int) toread, (unsigned int) (size - curread));
+	}
+	else
+		toread = CONTEXT_BLOCK_SIZE;
 
         LIBMTP_USB_DEBUG("Reading in 0x%04lx bytes\n", toread);
 
@@ -1194,13 +1207,13 @@ ptp_usb_sendreq(PTPParams* params, PTPContainer* req) {
 
 uint16_t
 ptp_usb_senddata(PTPParams* params, PTPContainer* ptp,
-        unsigned long size, PTPDataHandler *handler
+        uint64_t size, PTPDataHandler *handler
         ) {
     uint16_t ret;
     int wlen, datawlen;
     unsigned long written;
     PTPUSBBulkContainer usbdata;
-    uint32_t bytes_left_to_transfer;
+    uint64_t bytes_left_to_transfer;
     PTPDataHandler memhandler;
 
     LIBMTP_USB_DEBUG("SEND DATA PHASE\n");
@@ -1264,6 +1277,10 @@ static uint16_t ptp_usb_getpacket(PTPParams *params,
     PTPDataHandler memhandler;
     uint16_t ret;
     unsigned char *x = NULL;
+    unsigned long packet_size;
+    PTP_USB *ptp_usb = (PTP_USB *) params->data;
+
+    packet_size = ptp_usb->inep_maxpacket;
 
     /* read the header and potentially the first data */
     if (params->response_packet_size > 0) {
@@ -1277,7 +1294,7 @@ static uint16_t ptp_usb_getpacket(PTPParams *params,
         return PTP_RC_OK;
     }
     ptp_init_recv_memory_handler(&memhandler);
-    ret = ptp_read_func(PTP_USB_BULK_HS_MAX_PACKET_LEN_READ, &memhandler, params->data, rlen, 0);
+    ret = ptp_read_func(packet_size, &memhandler, params->data, rlen, 0);
     ptp_exit_recv_memory_handler(&memhandler, &x, rlen);
     if (x) {
         memcpy(packet, x, *rlen);
@@ -1334,7 +1351,7 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
                 break;
             }
         }
-        if (usbdata.length == 0xffffffffU) {
+        if (rlen == ptp_usb->inep_maxpacket) {
             /* Copy first part of data to 'data' */
             putfunc_ret =
                     handler->putfunc(
@@ -1350,7 +1367,7 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
                 uint16_t xret;
 
                 xret = ptp_read_func(
-                        PTP_USB_BULK_HS_MAX_PACKET_LEN_READ,
+                        0x20000000,
                         handler,
                         params->data,
                         &readdata,
@@ -1358,7 +1375,7 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
                         );
                 if (xret != PTP_RC_OK)
                     return xret;
-                if (readdata < PTP_USB_BULK_HS_MAX_PACKET_LEN_READ)
+                if (readdata < 0x20000000)
                     break;
             }
             return PTP_RC_OK;
@@ -1416,7 +1433,7 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
             return putfunc_ret;
 
         if (FLAG_NO_ZERO_READS(ptp_usb) &&
-                len + PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
+                len + PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket) {
 
             LIBMTP_USB_DEBUG("Reading in extra terminating byte\n");
 
@@ -1442,8 +1459,8 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
             xread = bulk.result.transferred_bytes;
 
             if (result != 1)
-                LIBMTP_INFO("Could not read in extra byte for PTP_USB_BULK_HS_MAX_PACKET_LEN_READ long file, return value 0x%04x\n", result);
-        } else if (len + PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ && params->split_header_data == 0) {
+                LIBMTP_INFO("Could not read in extra byte for %d bytes long file, return value 0x%04x\n", ptp_usb->inep_maxpacket, result);
+        } else if (len + PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket && params->split_header_data == 0) {
             int zeroresult = 0, xread;
             unsigned char zerobyte = 0;
 
@@ -1948,13 +1965,16 @@ static void close_usb(PTP_USB* ptp_usb) {
  * Self-explanatory?
  */
 static int find_interface_and_endpoints(openusb_dev_handle_t *dev,
+	uint8_t *conf,
         uint8_t *interface,
+        uint8_t *altsetting,
         int* inep,
         int* inep_maxpacket,
         int* outep,
         int *outep_maxpacket,
         int* intep) {
-    int i, ret;
+    uint8_t i;
+    int ret;
     struct usb_device_desc desc;
 
     ret = openusb_parse_device_desc(libmtp_openusb_handle, *dev, NULL, 0, &desc);
@@ -1967,6 +1987,7 @@ static int find_interface_and_endpoints(openusb_dev_handle_t *dev,
 
         ret = openusb_parse_config_desc(libmtp_openusb_handle, *dev, NULL, 0, i, &config);
         if (ret != OPENUSB_SUCCESS) continue;
+	*conf = desc.bConfigurationValue;
         // Loop over each configurations interfaces
         for (j = 0; j < config.bNumInterfaces; j++) {
             uint8_t k;
@@ -1983,6 +2004,7 @@ static int find_interface_and_endpoints(openusb_dev_handle_t *dev,
             if (no_ep != 3)
                 continue;
             *interface = ifcdesc.bInterfaceNumber;
+	    *altsetting = ifcdesc.bAlternateSetting;
             // Loop over the three endpoints to locate two bulk and
             // one interrupt endpoint and FAIL if we cannot, and continue.
             for (k = 0; k < no_ep; k++) {
@@ -2087,7 +2109,9 @@ LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
 
     /* Assign interface and endpoints to usbinfo... */
     err = find_interface_and_endpoints(ldevice,
+            &ptp_usb->conf,
             &ptp_usb->interface,
+            &ptp_usb->altsetting,
             &ptp_usb->inep,
             &ptp_usb->inep_maxpacket,
             &ptp_usb->outep,
