@@ -29,7 +29,8 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_ICONV
